package org.springblade.customer.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.customer.dto.FctCsIrregularDTO;
import org.springblade.customer.entity.FctCsIrregular;
import org.springblade.customer.mapper.FctCsIrregularMapper;
import org.springblade.customer.service.IFctCsIrregularService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 客服不规范定制数据表 服务实现类
 * </p>
 *
 * @author zbc
 * @since 2021-08-19
 */
@Service
@DS("oracle143")
public class FctCsIrregularServiceImpl extends ServiceImpl<FctCsIrregularMapper, FctCsIrregular> implements IFctCsIrregularService {

	@Autowired
	private FctCsIrregularMapper fctCsIrregularMapper;

	@Override
	public boolean insertBatch(List<FctCsIrregularDTO> excelList) {
		return fctCsIrregularMapper.insertBatch(excelList);
	}

	@Override
	public boolean deleteOldData(String sTime, String eTime,String spcode) {
		return fctCsIrregularMapper.deleteOldData(sTime,eTime,spcode);

	}

	@Override
	public void setSpid() {
		fctCsIrregularMapper.setSpid();
	}

	@Override
	public void setTheday() {
		fctCsIrregularMapper.setTheday();
	}

	@Override
	public void deleteInvisibleDatay(String sTime, String eTime) {
		fctCsIrregularMapper.deleteInvisibleDatay(sTime,eTime);
	}

	@Override
	public void deleteInvisibleDataw(String sTime, String eTime) {
		fctCsIrregularMapper.deleteInvisibleDataw(sTime,eTime);
	}

	@Override
	public void deleteInvisibleDatas(String sTime, String eTime) {
		fctCsIrregularMapper.deleteInvisibleDatas(sTime,eTime);
	}

	@Override
	public void callData1(String sTime) {
		fctCsIrregularMapper.callData1(sTime);
	}

	@Override
	public void callData12(String substring) {
		fctCsIrregularMapper.callData12(substring);
	}
}
